@@ -7,6 +7,8 @@ use Apache::Constants qw(OK DECLINED);
 use Apache::Connection ();
 use Apache::Server ();
 
+eval { require Apache::Table; };
+
 @Apache::EXPORT_OK = qw(exit warn);
 
 *import = \&Exporter::import;
@@ -18,20 +20,18 @@ if (caller eq "CGI::Apache") {
 
 {
     no strict;
-    $VERSION = "1.26";
+    $VERSION = "1.27";
     __PACKAGE__->mod_perl::boot($VERSION);
 }
 
-if($ENV{MOD_PERL} && perl_hook("Sections")) {
+BEGIN {
     *Apache::ReadConfig:: = \%ApacheReadConfig::;
+}
 
-    if(Apache::Constants::MODULE_MAGIC_NUMBER() >= 19971026) {
-	*Apache::httpd_conf = sub {
-	    shift;
-	    push @Apache::ReadConfig::PerlConfig,
-	    map "$_\n", @_;
-	};
-    }
+sub httpd_conf {
+    shift;
+    push @Apache::ReadConfig::PerlConfig,
+      map "$_\n", @_;
 }
 
 sub parse_args {
@@ -49,33 +49,39 @@ sub content {
     return unless $ct =~ m!^application/x-www-form-urlencoded!;
     my $buff;
     $r->read($buff, $r->header_in("Content-length"));
-    parse_args(wantarray, $buff);
+    return $buff unless wantarray;
+    parse_args(1, $buff);
 }
 
 sub args {
     my($r, $val) = @_;
-    parse_args(wantarray, 
-	       $val ? $r->query_string($val) : $r->query_string);
+    my $args = @_ > 1 ? $r->query_string($val) : $r->query_string;
+    return $args unless wantarray;
+    parse_args(1, $args);
 }
 
 *READ = \&read unless defined &READ;
 
 sub read {
-    my($r, $bufsiz) = @_[0,2];
+    my($r, $bufsiz, $offset) = @_[0,2,3];
     my($nrd, $buf, $total);
     $nrd = $total = 0;
     $buf = "";
-    $_[1] ||= "";
-    #$_[1] = " " x $bufsiz unless defined $_[1]; #XXX?
+    $_[1] = "" unless $offset;
 
-    $r->hard_timeout("Apache->read");
+    $r->soft_timeout("Apache->read");
 
     while($bufsiz) {
 	$nrd = $r->read_client_block($buf, $bufsiz) || 0;
 	if(defined $nrd and $nrd > 0) {
 	    $bufsiz -= $nrd;
-	    $_[1] .= $buf;
- 	    #substr($_[1], $total, $nrd) = $buf;
+            if ($offset) {
+                substr($_[1], $offset) .= $buf;
+                #$_[1] .= $buf;
+            }
+            else {
+                $_[1] .= $buf;
+            }
 	    $total += $nrd;
 	    next if $bufsiz;
 	    last;
@@ -113,7 +119,7 @@ sub new_read {
 	return 0;
     }
 
-    $r->hard_timeout("Apache->read");
+    $r->soft_timeout("Apache->read");
     
     while($bufsiz) {
 	$nrd = $r->get_client_block($buf, $bufsiz) || 0;
@@ -162,6 +168,7 @@ sub send_cgi_header {
     my($r, $headers) = @_;
     my $dlm = "\015?\012"; #a bit borrowed from LWP::UserAgent
     my($key, $val);
+    local $_;
     while(($_, $headers) = split /$dlm/, $headers, 2) {
 	#warn "hunk=`$_'\n";
 	#warn "rest=`$headers'\n";
@@ -179,18 +186,16 @@ sub send_cgi_header {
 	    else {
 		$not_sent = 1;
 	    }
-	    $r->send_http_header if $not_sent;
+            if ($not_sent) {
+                $r->send_http_header;
+                $r->sent_header(-2);
+            }
 	    $r->print($headers); #send rest of buffer, without stripping newlines!!!
 	    last;
 	}
     }
 }
 
-sub TIEHANDLE {
-    my($class, $r) = @_;
-    $r ||= Apache->request;
-}
-
 1;
 
 __END__
@@ -423,13 +428,16 @@ returned.  When called in a list context, a list of parsed I<key> =>
 I<value> pairs are returned.  *NOTE*: you can only ask for this once,
 as the entire body is read from the client.
 
-=item $r->read($buf, $bytes_to_read)
+=item $r->read($buf, $bytes_to_read, [$offset])
 
 This method is used to read data from the client, 
 looping until it gets all of C<$bytes_to_read> or a timeout happens.
 
+An offset may be specified to place the read data at some other place
+than the beginning of the string.
+
 In addition, this method sets a timeout before reading with
-C<$r-E<gt>hard_timeout>.
+C<$r-E<gt>soft_timeout>.
 
 =item $r->get_remote_host
 
@@ -687,6 +695,26 @@ This is the value of the User directive.
 Returns the numeric group id under which the server answers requests.
 This is the value of the Group directive.
 
+=item $s->loglevel
+
+Returns the value of the current LogLevel. This method is added by
+the Apache::Log module, which needs to be pulled in.
+
+    use Apache::Log;
+    print "LogLevel = ", $s->loglevel;
+
+If using Perl 5.005+, the following constants are defined (but not
+exported):
+
+    Apache::Log::EMERG
+    Apache::Log::ALERT
+    Apache::Log::CRIT
+    Apache::Log::ERR
+    Apache::Log::WARNING
+    Apache::Log::NOTICE
+    Apache::Log::INFO
+    Apache::Log::DEBUG
+
 =item $r->get_handlers( $hook )
 
 Returns a reference to a list of handlers enabled for $hook. $hook is
@@ -893,7 +921,21 @@ and the client should be told not to cache it.
 =item $r->print( @list )
 
 This method sends data to the client with C<$r-E<gt>write_client>, but first
-sets a timeout before sending with C<$r-E<gt>hard_timeout>.
+sets a timeout before sending with C<$r-E<gt>soft_timeout>. This method is
+called instead of CORE::print when you use print() in your mod_perl programs.
+
+This method treats scalar references specially. If an item in @list is a
+scalar reference, it will be dereferenced before printing. This is a
+performance optimization which prevents unneeded copying of large strings,
+and it is subtly different from Perl's standard print() behavior.
+
+Example:
+
+   $foo = \"bar"; print($foo);
+
+The result is "bar", not the "SCALAR(0xDEADBEEF)" you might have expected. If
+you really want the reference to be printed out, force it into a scalar
+context by using C<print(scalar($foo))>.
 
 =item $r->send_fd( $filehandle )
 
@@ -1086,6 +1128,20 @@ Returns true if the specified callback hook is enabled:
 
 =back
 
+=head1 GLOBAL VARIABLES
+
+=over 4
+
+=item $Apache::Server::Starting
+
+Set to true when the server is starting.
+
+=item $Apache::Server::ReStarting
+
+Set to true when the server is starting.
+
+=back
+
 =head1 SEE ALSO
 
 perl(1),
