@@ -5,8 +5,44 @@ use Exporter ();
 use IO::File ();
 use File::Copy ();
 
-*import = \&Exporter::import;
+$Apache::ExtUtils::VERSION = '1.04';
+
+my @config_export = qw(%Config ldopts ccopts);
 @Apache::ExtUtils::EXPORT = qw(command_table);
+@Apache::ExtUtils::EXPORT_OK = (qw(pm), @config_export);
+
+my $errsv = "";
+
+sub import {
+    my $class = shift;
+    my $config_export = join '|', @config_export;
+    for (@_) {
+	#perl -Mlib=lib -MApache::ExtUtils=%Config -e 'print $Config{cc}'
+	if (/$config_export/o) {
+	    require Config;
+	    *Apache::ExtUtils::Config = \%Config::Config;
+	    Config_pm_fixup();
+	    require ExtUtils::Embed;
+	}
+    }
+    local $Exporter::ExportLevel = 1;
+    $class->Exporter::import(@_);
+}
+
+*ldopts = \&ExtUtils::Embed::ldopts;
+*ccopts = \&ExtUtils::Embed::ccopts;
+
+sub Config_pm_fixup {
+    my %config_fixups = (
+       ccdlflags => sub { s/-R\s+/-R/; },
+    );
+
+    while (my($key, $sub) = each %config_fixups) {
+	local $_ = $Config::Config{$key};
+	$sub->();
+	(tied %Config::Config)->{$key} = $_;
+    }
+}
 
 sub command_table {
     my($class, $cmds);
@@ -23,12 +59,18 @@ sub command_table {
 	require "$file.pm"; #so we can see prototypes
     };
     if ($@) {
+	unless ($@ =~ /Can.t locate /) {
+	    $errsv = $@;
+	}
 	require ExtUtils::testlib;
         ExtUtils::testlib->import;
 	require lib;
 	my $lib = "lib";#hmm, lib->import + -w == Unquoted string "lib" ...
 	$lib->import('./lib');
 	eval { require $class };
+	if ($@ and $@ !~ /Can.t locate /) {
+	    $errsv ||= $@;
+	}
     }
     unless (-e "$file.xs.orig") {
         File::Copy::cp("$file.xs", "$file.xs.orig");
@@ -117,7 +159,10 @@ sub xs_cmd_table {
 	    }
 	}
 	$desc ||= "1-3 value(s) for $name";
-
+	unless ($args_how) {
+	    $errsv ||= $@;
+	    die "Can't determine prototype for `$sub': $errsv";
+	}
 	(my $cname = $name) =~ s/\W/_/g;
 	$infos .= cmd_info($cname, $sub, $cmd_data, $args_how);
 	$cmdtab .= <<EOF;
@@ -149,7 +194,7 @@ static mod_perl_perl_dir_config *newPerlConfig(pool *p)
 	(mod_perl_perl_dir_config *)
 	    palloc(p, sizeof (mod_perl_perl_dir_config));
     cld->obj = Nullsv;
-    cld->class = "$class";
+    cld->pclass = "$class";
     register_cleanup(p, cld, perl_perl_cmd_cleanup, null_cleanup);
     return cld;
 }
@@ -203,14 +248,104 @@ module MODULE_VAR_EXPORT XS_${modname} = {
 
 MODULE = $class		PACKAGE = $class
 
+PROTOTYPES: DISABLE
+
 BOOT:
     XS_${modname}.name = "$class";
     add_module(&XS_${modname});
     stash_mod_pointer("$class", &XS_${modname});
 
+void
+END()
+
+    CODE:
+    if (find_linked_module("$class")) {
+        remove_module(&XS_${modname});
+    }
 EOF
 }
 
+#perl -MApache::ExtUtils=pm -e pm -- Apache::Foo
+sub pm {
+    my($class) = @_ ? @_ : @ARGV;
+    (my $name = $class) =~ s/.*::(\w+)$/$1/;
+    write_pm($class, $name);
+    write_makepl($class, $name);
+}
+
+sub outfh {
+    my($file) = @_;
+
+    my $fh = local *FH;
+    if (-e $file) {
+	die "$file exists";
+    }
+    open $fh, ">$file" or die "open $file: $!";
+    print STDERR "writing $file\n";
+    return $fh;
+}
+
+sub write_pm {
+    my($class, $name) = @_;
+    my $fh = outfh("$name.pm");
+    print $fh <<EOF;
+package $class;
+
+use strict;
+use Apache::ModuleConfig ();
+use DynaLoader ();
+ 
+if(\$ENV{MOD_PERL}) {
+    no strict;
+    \$VERSION = '1.00';
+    \@ISA = qw(DynaLoader);
+     __PACKAGE__->bootstrap(\$VERSION);
+}
+
+sub DirectiveName (\$\$\$) {
+     my(\$cfg, \$parms, \$arg) = \@_;
+     my \$scfg = Apache::ModuleConfig->get(\$parms->server);
+
+}
+
+1;
+__END__
+EOF
+  close $fh or die $!;
+}
+
+sub write_makepl {
+    my($class, $name) = @_;
+
+    my $fh = outfh("Makefile.PL");
+    print $fh <<EOF;
+package $class;
+
+use ExtUtils::MakeMaker;
+
+use Apache::ExtUtils qw(command_table);
+use Apache::src ();
+
+my \@directives = ( 
+	 	   { 
+		    name     => 'DirectiveName',
+		    errmsg   => 'the syntax error message',
+		    args_how => 'TAKE1',
+		    req_override => 'OR_ALL',
+		   }
+		  );
+
+command_table(\\\@directives);
+
+WriteMakefile(
+     'NAME'	=> __PACKAGE__,
+     'VERSION_FROM' => '$name.pm',
+     'INC'	=> Apache::src->new->inc,
+ );
+EOF
+  close $fh or die $!;
+}
+
 1;
 
 __END__
