@@ -2,7 +2,11 @@ package Apache::PerlRun;
 
 use strict;
 use vars qw($Debug);
-use Apache::Constants qw(:common OPT_EXECCGI);
+use Apache::Constants qw(:common &OPT_EXECCGI);
+
+BEGIN {
+    OPT_EXECCGI() if $ENV{MOD_PERL}; #preload, :common are alread pre-loaded
+}
 
 unless (defined $Apache::Registry::NameWithVirtualHost) {
     $Apache::Registry::NameWithVirtualHost = 1;
@@ -15,32 +19,22 @@ unless (defined $Apache::Registry::MarkLine) {
 $Debug ||= 0;
 my $Is_Win32 = $^O eq "MSWin32";
 
-@Apache::PerlRun::ISA = qw(Apache);
-
 sub new {
     my($class, $r) = @_;
-    return $r unless ref($r) eq "Apache";
-    if(ref $r) {
-	$r->request($r);
-    }
-    else {
-	$r = Apache->request;
-    }
     my $filename = $r->filename;
     $r->warn("Apache::PerlRun->new for $filename in process $$")
 	if $Debug && $Debug & 4;
 
-    bless {
-	'_r' => $r,
-    }, $class;
+    return bless {r=>$r}, $class;
 }
 
 sub can_compile {
     my($pr) = @_;
-    my $filename = $pr->filename;
-    if (-r $filename && -s _) {
-	if (!($pr->allow_options & OPT_EXECCGI)) {
-	    $pr->log_reason("Options ExecCGI is off in this directory",
+    my $r = $pr->{r};
+    my $filename = $r->filename;
+    if (-r $r->finfo && -s _) {
+	if (!($r->allow_options & OPT_EXECCGI)) {
+	    $r->log_reason("Options ExecCGI is off in this directory",
 			   $filename);
 	    return FORBIDDEN;
  	}
@@ -48,7 +42,7 @@ sub can_compile {
 	    return DECLINED;
 	}
 	unless (-x _ or $Is_Win32) {
-	    $pr->log_reason("file permissions deny server execution",
+	    $r->log_reason("file permissions deny server execution",
 			   $filename);
 	    return FORBIDDEN;
 	}
@@ -56,12 +50,12 @@ sub can_compile {
 	$pr->{'mtime'} = -M _;
 	return wantarray ? (OK, $pr->{'mtime'}) : OK;
     }
+    $pr->log_error("$filename not found or unable to stat");
     return NOT_FOUND;
 }
 
 sub mark_line {
-    my($pr) = @_;
-    my $filename = $pr->filename;
+    my $filename = shift->{r}->filename;
     return $Apache::Registry::MarkLine ?
 	"\n#line 1 $filename\n" : "";
 }
@@ -100,7 +94,7 @@ sub should_compile {
       $Apache::Registry->{$package}{'mtime'} <= $mtime);
 }
 
-sub update_mtime {
+sub set_mtime {
     my($pr, $mtime, $package) = @_;
     $mtime   ||= $pr->{'mtime'};
     $package ||= $pr->{'namespace'};
@@ -110,35 +104,40 @@ sub update_mtime {
 sub compile {
     my($pr, $eval) = @_;
     $eval ||= $pr->{'sub'};
-    $pr->clear_rgy_endav;
-    $pr->log_error("Apache::PerlRun->compile") if $Debug && $Debug & 4;
+    my $r = $pr->{r};
+    $r->clear_rgy_endav;
+    $r->log_error("Apache::PerlRun->compile") if $Debug && $Debug & 4;
     Apache->untaint($$eval);
     {
 	no strict; #so eval'd code doesn't inherit our bits
 	eval $$eval;
     }
-    $pr->stash_rgy_endav;
+    $r->stash_rgy_endav;
     return $pr->error_check;
 }
 
 sub run {
     my $pr = shift;
     my $package = $pr->{'namespace'};
+    my $r = $pr->{r};
 
     my $rc = OK;
     my $cv = \&{"$package\::handler"};
-    eval { $rc = &{$cv}($pr, @_) } if $pr->seqno;
+
+    my $oldwarn = $^W;
+    eval { $rc = &{$cv}($r, @_) } if $r->seqno;
     $pr->{status} = $rc;
+    $^W = $oldwarn;
 
     my $errsv = "";
     if($@) {
 	$errsv = $@;
 	$@ = ''; #XXX fix me, if we don't do this Apache::exit() breaks
-	$@{$pr->uri} = $errsv;
+	$@{$r->uri} = $errsv;
     }
 
     if($errsv) {
-	$pr->log_error($errsv);
+	$r->log_error($errsv);
 	return SERVER_ERROR;
     }
 
@@ -146,43 +145,55 @@ sub run {
 }
 
 sub status {
-    shift->{'_r'}->status;
+    shift->{r}->status;
+}
+
+sub namespace_from {
+    my($pr) = @_;
+    my $r = $pr->{r};
+
+    my $uri = $r->uri;
+
+    $r->log_error(sprintf "Apache::PerlRun->namespace escaping %s",
+		  $uri) if $Debug && $Debug & 4;
+
+    my $path_info = $r->path_info;
+    my $script_name = $path_info && $uri =~ /$path_info$/ ?
+	substr($uri, 0, length($uri)-length($path_info)) :
+	$uri;
+
+    if ($Apache::Registry::NameWithVirtualHost && $r->server->is_virtual) {
+	my $name = $r->get_server_name;
+	$script_name = join "", $name, $script_name if $name;
+    }
+
+    $script_name =~ s:/+$:/__INDEX__:;
+
+    return $script_name;
 }
 
 sub namespace {
     my($pr, $root) = @_;
     return $pr->{'namespace'} if $pr->{'namespace'};
 
-    my $uri = $pr->uri; 
-    $uri = "/__INDEX__" if $uri eq "/";
-    $pr->log_error(sprintf "Apache::PerlRun->namespace escaping %s",
-		  $uri) if $Debug && $Debug & 4;
-
-    my $script_name = $pr->path_info ?
-	substr($uri, 0, length($uri)-length($pr->path_info)) :
-	    $uri;
-
-    if($Apache::Registry::NameWithVirtualHost) {
-	my $name = $pr->get_server_name;
-	$script_name = join "", $name, $script_name if $name;
-    }
+    my $script_name = $pr->namespace_from;
 
     # Escape everything into valid perl identifiers
-    $script_name =~ s/([^A-Za-z0-9\/])/sprintf("_%2x",unpack("C",$1))/eg;
+    $script_name =~ s/([^A-Za-z0-9_\/])/sprintf("_%2x",unpack("C",$1))/eg;
 
     # second pass cares for slashes and words starting with a digit
     $script_name =~ s{
 			  (/+)       # directory
 			  (\d?)      # package's first character
 			 }[
-			   "::" . ($2 ? sprintf("_%2x",unpack("C",$2)) : "")
+			   "::" . (length $2 ? sprintf("_%2x",unpack("C",$2)) : "")
 			  ]egx;
 
     $Apache::Registry::curstash = $script_name;
  
     $root ||= "Apache::ROOT";
 
-    $pr->log_error("Apache::PerlRun->namespace: package $root$script_name")
+    $pr->{r}->log_error("Apache::PerlRun->namespace: package $root$script_name")
 	if $Debug && $Debug & 4;
 
     $pr->{'namespace'} = $root.$script_name;
@@ -191,13 +202,13 @@ sub namespace {
 
 sub readscript {
     my $pr = shift;
-    $pr->{'code'} = $pr->slurp_filename;
+    $pr->{'code'} = $pr->{r}->slurp_filename;
 }
 
 sub error_check {
     my $pr = shift;
     if ($@ and substr($@,0,4) ne " at ") {
-	$pr->log_error("PerlRun: `$@'");
+	$pr->{r}->log_error("PerlRun: `$@'");
 	$@{$pr->uri} = $@;
 	$@ = ''; #XXX fix me, if we don't do this Apache::exit() breaks	
 	return SERVER_ERROR;
@@ -239,17 +250,24 @@ sub parse_cmdline {
 
 sub chdir_file {
     my($pr, $dir) = @_;
-    $pr->{'_r'}->chdir_file($dir ? $dir : $pr->filename);
+    my $r = $pr->{r};
+    $r->chdir_file($dir ? $dir : $r->filename);
 }
 
 sub set_script_name {
-    my($pr) = @_;
-    *0 = \$pr->filename;
+    *0 = \(shift->{r}->filename);
 }
 
-sub handler {
-    my $r = shift;
-    my $pr = Apache::PerlRun->new($r);
+sub handler ($$) {
+    my($class, $r);
+    if (@_ >= 2) {
+	($class, $r) = (shift, shift);
+    }
+    else {
+	($class, $r) = (__PACKAGE__, shift);
+    }
+
+    my $pr = $class->new($r);
     my $rc = $pr->can_compile;
     return $rc unless $rc == OK;
 
@@ -282,18 +300,57 @@ sub handler {
 	$r->child_terminate if lc($opt) eq "on";
     }
 
-    {   #flush the namespace
-	no strict;
-	my $tab = \%{$package.'::'};
-        foreach (keys %$tab) {
-	    undef &{$tab->{$_}} if defined &{$tab->{$_}};
-	}
-	%$tab = ();
-    }
+    $pr->flush_namespace($package);
 
     return $rc;
 }
 
+BEGIN {
+    if ($] < 5.006) {
+        $INC{'warnings.pm'} = __FILE__;
+        *warnings::unimport = sub {};
+    }
+}
+
+sub flush_namespace {
+    my($self, $package) = @_;
+    $package ||= $self->namespace;
+
+    no strict 'refs';
+    my $tab = \%{$package.'::'};
+
+    for (keys %$tab) {
+        my $fullname = join '::', $package, $_;
+        #code/hash/array/scalar might be imported
+        #make sure the gv does not point elsewhere
+        #before undefing each
+        if (%$fullname) {
+            *{$fullname} = {};
+            undef %$fullname;
+        }
+        if (@$fullname) {
+            *{$fullname} = [];
+            undef @$fullname;
+        }
+        if ($$fullname) {
+            my $tmp; #argh, no such thing as an anonymous scalar
+            *{$fullname} = \$tmp;
+            undef $$fullname;
+        }
+        if (defined &$fullname) {
+            no warnings;
+            local $^W = 0;
+            *{$fullname} = sub {};
+	    undef &$fullname;
+	}
+        if (*{$fullname}{IO}) {
+            if (fileno $fullname) {
+                close $fullname;
+            }
+        }
+    }
+}
+
 1;
 
 __END__
@@ -354,3 +411,4 @@ perl(1), mod_perl(3), Apache::Registry(3)
 
 Doug MacEachern
 
+=cut
