@@ -1,3 +1,52 @@
+/* ====================================================================
+ * The Apache Software License, Version 1.1
+ *
+ * Copyright (c) 1996-2000 The Apache Software Foundation.  All rights
+ * reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The end-user documentation included with the redistribution,
+ *    if any, must include the following acknowledgment:
+ *       "This product includes software developed by the
+ *        Apache Software Foundation (http://www.apache.org/)."
+ *    Alternately, this acknowledgment may appear in the software itself,
+ *    if and wherever such third-party acknowledgments normally appear.
+ *
+ * 4. The names "Apache" and "Apache Software Foundation" must
+ *    not be used to endorse or promote products derived from this
+ *    software without prior written permission. For written
+ *    permission, please contact apache@apache.org.
+ *
+ * 5. Products derived from this software may not be called "Apache",
+ *    nor may "Apache" appear in their name, without prior written
+ *    permission of the Apache Software Foundation.
+ *
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
+ * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
+ * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+ * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
+ * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ * ====================================================================
+ */
+
 #ifdef WIN32
 #define NO_PERL_CHILD_INIT
 #define NO_PERL_CHILD_EXIT
@@ -53,7 +102,15 @@
 #include "perl_PL.h"
 #endif
 #else
+#define PERL_PATCHLEVEL_H_IMPLICIT /* ignore local_patches */
 #include "patchlevel.h"
+#undef PERL_PATCHLEVEL_H_IMPLICIT
+#ifndef PATCHLEVEL
+#define PATCHLEVEL PERL_VERSION
+#undef  SUBVERSION
+#define SUBVERSION PERL_SUBVERSION
+#endif
+
 #if ((PATCHLEVEL >= 4) && (SUBVERSION >= 76)) || (PATCHLEVEL >= 5)
 #include "perl_PL.h"
 #endif
@@ -100,13 +157,15 @@
 #undef die
 #undef __attribute__
 
+#ifdef pTHX_
+#define PERL_IS_5_6
+#endif
+
 #ifndef _INCLUDE_APACHE_FIRST
 #include "apache_inc.h"
 #endif
 
-#ifdef pTHX_
-#define PERL_IS_5_6
-#else
+#ifndef PERL_IS_5_6
 #define pTHX_
 #define aTHXo_
 #define CopFILEGV(cop) cop->cop_filegv
@@ -292,6 +351,11 @@ mp_magic_setenv(key, val, 0)
 extern U32	mp_debug;
 
 #ifdef PERL_TRACE
+
+/* -Wall */
+#undef dNOOP
+#define dNOOP extern int __attribute__ ((unused)) Perl___notused
+
 #define MP_TRACE(a)   if (mp_debug)	 a
 #define MP_TRACE_d(a) if (mp_debug & 1)	 a /* directives */
 #define MP_TRACE_s(a) if (mp_debug & 2)	 a /* perl sections */
@@ -382,7 +446,7 @@ if((add->flags & f) || (base->flags & f)) \
 #define MP_ENV_off(d)    (d->flags  &= ~MPf_ENV)
 #endif
 
-#define MP_ENV(d)       (d->SetupEnv == MPf_On)
+#define MP_ENV(d)       (d->SetupEnv != MPf_Off)
 #define MP_ENV_on(d)    (d->SetupEnv = MPf_On)
 #define MP_ENV_off(d)   (d->SetupEnv = MPf_Off)
 
@@ -1081,7 +1145,7 @@ request_rec *perl_request_rec(request_rec *);
 void perl_setup_env(request_rec *r);
 SV  *perl_bless_request_rec(request_rec *); 
 void perl_set_request_rec(request_rec *); 
-void mod_perl_cleanup_av(void *data);
+void mod_perl_cleanup_sv(void *data);
 void mod_perl_cleanup_handler(void *data);
 void mod_perl_end_cleanup(void *data);
 void mod_perl_register_cleanup(request_rec *r, SV *sv);
@@ -1107,12 +1171,10 @@ void perl_stash_rgy_endav(char *s, SV *rgystash);
 void perl_run_rgy_endav(char *s);
 void perl_run_endav(char *s);
 void perl_call_halt(int status);
-CV *empty_anon_sub(void);
 void perl_reload_inc(server_rec *s, pool *p);
 I32 perl_module_is_loaded(char *name);
 SV *perl_module2file(char *name);
 int perl_require_module(char *module, server_rec *s);
-void perl_qrequire_module (char *name);
 int perl_load_startup_script(server_rec *s, pool *p, char *script, I32 my_warn);
 array_header *perl_cgi_env_init(request_rec *r);
 void perl_clear_env(void);
@@ -1136,6 +1198,7 @@ void perl_stdout2client(request_rec *r);
     perl_require_module("Apache", s)
 
 char *mod_perl_auth_name(request_rec *r, char *val);
+char *mod_perl_auth_type(request_rec *r, char *val);
 
 module *perl_get_module_ptr(char *name, int len);
 void *perl_merge_server_config(pool *p, void *basev, void *addv);
