@@ -1,10 +1,20 @@
-#! /usr/local/bin/perl
+#!perl
+
+unless (defined $ENV{MOD_PERL}) {
+    die "\$ENV{MOD_PERL} not set!";
+}
+
 BEGIN {
     #./blib/lib:./blib/arch
     use ExtUtils::testlib;
 
     use lib map { "$Apache::Server::CWD/$_" } qw(t/docs blib/lib blib/arch);
     require "blib.pl" if -e "./t/docs/blib.pl";
+    #Perl ignores w/ -T
+    if ($ENV{PERL5LIB} and $ENV{PASS_PERL5LIB}) {
+         unshift @INC, map { Apache->untaint($_) } split ":", $ENV{PERL5LIB};
+    }
+
     $Apache::Server::Starting or warn "Server is not starting !?\n";
     \$Apache::Server::Starting == \$Apache::ServerStarting or 
 	warn "GV alias broken\n";
@@ -12,6 +22,12 @@ BEGIN {
 	warn "GV alias broken\n";
 }
 
+if ($] >= 5.005 and -e "t/docs/local.pl") {
+    eval {
+	require "local.pl"; 
+    }; $@='' if $@;
+}
+
 # BSD/OS 3.1 gets confused with some dynamically loaded code inside evals,
 # so make sure IO::File is loaded here, rather than later within an eval.
 # this should not harm any other platforms, since IO::File will be used
@@ -26,8 +42,6 @@ unless ($INC{'Apache.pm'} =~ /blib/) {
 
 Apache::Constants->export(qw(HTTP_MULTIPLE_CHOICES));
 
-#no mod_perl qw(Connection Server);
-
 eval {
     require Apache::PerlRunXS;
 }; $@ = '' if $@;
@@ -91,6 +105,11 @@ sub main::access { print ++$Access::Cnt }
 
 $ENV{GATEWAY_INTERFACE} =~ /^CGI-Perl/ or die "GATEWAY_INTERFACE not set!";
 
+sub Outside::code {4}
+%Outside::hash = (one => 1);
+@Outside::array = qw(one);
+$Outside::scalar = 'one';
+
 #will be redef'd during tests
 sub PerlTransHandler::handler {-1}
 
@@ -154,9 +173,9 @@ sub Apache::AuthenTest::handler {
     my($res, $sent_pwd) = $r->get_basic_auth_pw;
     return $res if $res; #decline if not Basic
 
-    my $user = lc $r->connection->user;
+    my $user = lc $r->user;
     $r->notes("DoAuthenTest", 1);
-
+    
     unless($user eq "dougm" and $sent_pwd eq "mod_perl") {
         $r->note_basic_auth_failure;
         return AUTH_REQUIRED;
